<?php
	/*=======================================================================
	 * copyright (c) AL-Design Works 
	 * http://cms.al-design.jp/
	 * このソースファイルは CMS Designer の一部です。
	 * 詳しいライセンス規約についてはウェブサイトを参照してください。
	 *=======================================================================
	 */

	//require_once( dirname(__FILE__) . "/app.php.inc" );
	require_once( dirname(__FILE__) . "/manager.util.php.inc" );
	
	require_once( dirname(__FILE__) . "/manager.base.php.inc" );

	/* --------------------------------------------------------------------
	 * 日付入力項目クラス。date, datetime, time, year, month, entrydate。
	 * 
	 */ 
	class DateEntryItem extends BaseEntryItem
	{
		
		/* ----------------------------------------------------------------
		 * コンストラクタ。
		 * $xmlSchemaData : 対応するスキーマDataのXML要素。
		 */
		function DateEntryItem( $xmlSchemaData, $entryName, $entryId  )
		{
			// ベースコンストラクタの呼び出し。
			parent::BaseEntryItem( $xmlSchemaData, $entryName, $entryId  );
		}
		
		/* ----------------------------------------------------------------
		 * 当項目のXMLノードを新規作成する。
		 * $xmlDocParentEntry : エントリのDOMオブジェクト。
		 * $setInitData : 初期値。trueが指定されていた場合、スキーマで指定された値が設定される。デフォルト値はtrue。
		 * return : 新規XMLノード。
		 */
		function createNew( $xmlDocParentEntry, $setInitData = true )
		{
   			$newNode = parent::createNew( $xmlDocParentEntry, false ); // 現状、日付型は初期値を指定できない（指定しても無視される）。
			
			$this->setYMDHNS( $newNode, date( "Y" ), date( "m" ), date( "d" ), date( "H" ), date( "i" ), date( "s" ) );
			
			return $newNode;
		}
		
		function createEntryDate( &$xmlEntry )
		{
			$this->setYMDHNS( $xmlEntry, date( "Y" ), date( "m" ), date( "d" ), date( "H" ), date( "i" ), date( "s" ) );
		}
		
		function setYMDHNS( &$xmlNewNode, $year, $month, $day, $hour, $minute, $second )
		{
			// 年月日時分秒
			$sYear = sprintf( "%04d", $year );
			$sMonth = sprintf( "%02d", $month );
			$sDay = sprintf( "%02d", $day );
			$sHour = sprintf( "%02d", $hour );
			$sMinute = sprintf( "%02d", $minute );
			$sSecond = sprintf( "%02d", $second );
			
			// time値の取得
			$time = mktime( intval($sHour), intval($sMinute), intval($sSecond), intval($sMonth), intval($sDay), intval($sYear) );
			
			// 曜日を計算する。
			$arrWeekday = array( 'sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat' );
			$iWeekday = date('w', $time );
			$sWeekday = $arrWeekday[ $iWeekday ];
			
			if ( $this->dataType == 'entrydate' )
			{
				$strDate = $sYear . '-' . $sMonth . '-' . $sDay . ' ' . $sHour . ':' . $sMinute;
				$xmlNewNode->set_attribute( 'date', $strDate );
			}
			else
			{
				EntryUtil::deleteTextNode( $xmlNewNode );
				
				switch( $this->dataType )
				{
					case 'datetime' :
						$strDate = $sYear . '-' . $sMonth . '-' . $sDay . ' ' . $sHour . ':' . $sMinute;
						break;
					
					case 'date' :
						$strDate = $sYear . '-' . $sMonth . '-' . $sDay;
						break;
					
					case 'time' :
						$strDate = $sHour . ':' . $sMinute;
						break;
					
					case 'year' :
						$strDate = $sYear;
						break;
					
					case 'month' :
						$strDate = $sYear . '-' . $sMonth;
						break;
					
				}
				
				$xmlNewNode->set_content( $strDate );
			}
			
			$xmlNewNode->set_attribute( 'year', $sYear );
			$xmlNewNode->set_attribute( 'month', $sMonth );
			$xmlNewNode->set_attribute( 'day', $sDay );
			$xmlNewNode->set_attribute( 'hour', $sHour );
			$xmlNewNode->set_attribute( 'minute', $sMinute );
			$xmlNewNode->set_attribute( 'second', $sSecond );	
			$xmlNewNode->set_attribute( 'weekday', $sWeekday );
			$xmlNewNode->set_attribute( 'time', $time );
		}
		
		
		/* ----------------------------------------------------------------
		 * 入力内容でエントリ項目を更新する。
		 * $xpathEntry : エントリのXPathオブジェクト。
		 * $currentEntry : 現在のエントリ位置を表すXPath文字列。
		 */
		function update( $xpathEntry, $currentEntry )
		{
			
			// エントリXMLを取得。
			/// エントリ更新日時(entrydate)の場合、entry要素自身を取得して更新する。
			if ( $this->dataType == 'entrydate' )
			{
				$xmlEntryDataArr = $xpathEntry->xpath_eval( $currentEntry );
				
				// エントリ更新日時(entrydate)の場合さらに、「現在日時」フラグが立っていたら今日の日時を設定して終了する。
				if ( isset( $_POST[ 'cmsd_date_autoupdate' ] ) )
				{
					$this->setYMDHNS( $xmlEntryDataArr->nodeset[0], date( "Y" ), date( "m" ), date( "d" ), date( "H" ), date( "i" ), date( "s" ) );
					return true;
				}
			}
			else
			{
				$xmlEntryDataArr = $xpathEntry->xpath_eval( $currentEntry . '/' . $this->dataName );
			}
   			$xmlEntryData = $xmlEntryDataArr->nodeset[0];
			
			
			// 初期化
			$year = '';
			$month = '';
			$day = '';
			$hour = '';
			$minute = '';
			$second = '';
						
			// year
			if ( in_array( $this->dataType, array( 'entrydate', 'datetime', 'date', 'year', 'month' ) ) )
			{
				$year = $_POST[ $this->dataName . '_year' ];
				
				// 範囲チェック
				$iYear = intval( $year ); 
				if ( $iYear < 0 or $iYear > 9999 )
				{
					$this->errorMessage = sprintf( "%s(年) が 0-9999の範囲を超えました", $this->dataCaption );
					return false;
				}
				
			}
			
			// month
			if ( in_array( $this->dataType, array( 'entrydate', 'datetime', 'date', 'month' ) ) )
			{
				$month = $_POST[ $this->dataName . '_month' ];
				
				// 範囲チェック
				$iMonth = intval( $month ); 
				if ( $iMonth < 0 or $iMonth > 12 )
				{
					$this->errorMessage = sprintf( "%s(月) が 0-12の範囲を超えました", $this->dataCaption );
					return false;
				}
				
			}

			// day
			if ( in_array( $this->dataType, array( 'entrydate', 'datetime', 'date' ) ) )
			{
				$day = $_POST[ $this->dataName . '_day' ];
				
				// 範囲チェック
				$iDay = intval( $day ); 
				if ( $iDay < 0 or $iDay > 31 )
				{
					$this->errorMessage = sprintf( "%s(日) が 0-31の範囲を超えました", $this->dataCaption );
					return false;
				}
			}
			
			// hour
			if ( in_array( $this->dataType, array( 'entrydate', 'datetime', 'time' ) ) )
			{
				$hour = $_POST[ $this->dataName . '_hour' ];
				
				// 範囲チェック
				$iHour = intval( $hour ); 
				if ( $iHour < 0 or $iHour > 23 )
				{
					$this->errorMessage = sprintf( "%s(時) が 0-24の範囲を超えました", $this->dataCaption );
					return false;
				}
			}

			// minute
			if ( in_array( $this->dataType, array( 'entrydate', 'datetime', 'time' ) ) )
			{
				$minute = $_POST[ $this->dataName . '_minute' ];
				
				// 範囲チェック
				$iMinute = intval( $minute ); 
				if ( $iMinute < 0 or $iMinute > 59 )
				{
					$this->errorMessage = sprintf( "%s(時) が 0-59の範囲を超えました", $this->dataCaption );
					return false;
				}
			}
									
			$this->setYMDHNS( $xmlEntryData, $year, $month, $day, $hour, $minute, $second );
			
			return true;
		
		}
 
		
		/* ----------------------------------------------------------------
		 * フォーム項目を描画するHTMLを返す。
		 * $xpathEntry : エントリのXPathオブジェクト。
		 * $currentEntry : 現在のエントリ位置を表すXPath文字列。
		 */
		function renderFormItemXML( $xpathEntry, $currentEntry, &$xmlOutputEntryData )
		{
			if ( $this->dataType == 'entrydate' )
			{
				// 現在の階層を取得（entry要素を取得）。
				$xmlEntryDataArr = $xpathEntry->xpath_eval( $currentEntry );
			}
			else
			{
				// 現在の階層の対象データ項目を取得
				$xmlEntryDataArr = $xpathEntry->xpath_eval( $currentEntry . '/' . $this->dataName );
			}
			
   			$xmlEntryData = $xmlEntryDataArr->nodeset[0];

			
			$year = $xmlEntryData->get_attribute('year');
			$month = $xmlEntryData->get_attribute('month');
			$day = $xmlEntryData->get_attribute('day');
			$hour = $xmlEntryData->get_attribute('hour');
			$minute = $xmlEntryData->get_attribute('minute');
			$second = $xmlEntryData->get_attribute('second');
			
			$xmlOutputEntryData->set_attribute('year', $year );
			$xmlOutputEntryData->set_attribute('month', $month );
			$xmlOutputEntryData->set_attribute('day', $day );
			$xmlOutputEntryData->set_attribute('hour', $hour );
			$xmlOutputEntryData->set_attribute('minute', $minute );
			$xmlOutputEntryData->set_attribute('second', $second );
			
		}
		
	
	}

?>