<?php

	/*=======================================================================
	 * copyright (c) AL-Design Works 
	 * http://cms.al-design.jp/
	 * このソースファイルは CMS Designer の一部です。
	 * 詳しいライセンス規約についてはウェブサイトを参照してください。
	 *=======================================================================
	 */

	//require_once( dirname(__FILE__) . "/app.php.inc" );
	//require_once( dirname(__FILE__) . "/xmlutil.php.inc" );
	//require_once( dirname(__FILE__) . "/user.php.inc" );	// 2005.07.28 追加。非公開データの閲覧の為。
	require_once( dirname(__FILE__) . "/viewcore.php.inc" );
	require_once( dirname(__FILE__) . "/cache.php.inc" );
	
	// セッション開始
	cmsd_session_start();

	mb_language( 'Japanese' );
	mb_internal_encoding( gCMSDInternalEncoding() );	// 2015.08.27 UTF-8化対応

	//set_error_handler( "error_handler" );
  	//ob_start("execute_cmsd_command");
	ob_start();
	
	function cmsd_end_template()
	{
		// ブラウザキャッシュとサーバキャッシュの比較（If-Modified-Since対応）
		$cacheID = ADPageCache::createCacheIDFromRequest();
		$etag = md5( $cacheID );	// EtagはURLから生成
		
		// Expiresヘッダ用のブラウザキャッシュ時間。
		/// この間、If-Modified-Sinceの問い合わせも無くなるケースがあり（ブラウザによる）、あまり長くするとコンテンツ更新がユーザーに届くのが遅くなる。
		/// 最長でも1日(3600*24)ぐらいにしておくこと。
		$cacheInterval = 60; // [sec]
		
		// キャッシュが有効かつ存在し、有効期限内であれば、それを代わりに出力して終了する。
		if ( ADPageCache::isEnabled() && ADPageCache::exists( $cacheID ) && ! ADPageCache::isExpired( $cacheID ) )
		{
			// 出力バッファを取得し、一旦出力をクリアする。
			$buffer = ob_get_contents();
			ob_end_clean();
			
			// ブラウザキャッシュよりサーバキャッシュの更新日時の方が古ければ、$modifiedをfalse（未更新）にする。
			$timestamp = filemtime( ADPageCache::getCacheFilePath( $cacheID ) );
			$sLastModified = gmdate('D, d M Y H:i:s T', $timestamp);	
			$modified = true;
			
			if( isset( $_SERVER['HTTP_IF_MODIFIED_SINCE'] ) ) {
				$since = parse_http_date( $_SERVER['HTTP_IF_MODIFIED_SINCE'] );
				if( $since["timestamp"] >= $timestamp ) {
					$modified = false;
				}
			}
			
			// キャッシュが未更新なら、304 Not Modifiedを返して終了
			if ( ! $modified )
			{
				// PHPのデフォルト設定でno-cacheが送られてしまうようなので、ここでExpiresやCache-control、Pragmaを上書きする。
				
				header( "HTTP/1.1 304 Not Modified" );
				
				header( "Expires: " . gmdate( "D, d M Y H:i:s", time() + $cacheInterval ) . " GMT" );
				header( "Cache-Control: max-age=" . $cacheInterval);
				header( "Pragma: cache"); // no-cache以外を設定
				
				header( "Etag: \"$etag\"" );
				header( "Last-Modified: " . $sLastModified );
				return;		// ここで終了
			}
			else
			{
				header( "Expires: " . gmdate( "D, d M Y H:i:s", time() + $cacheInterval ) . " GMT" );
				header( "Cache-Control: max-age=" . $cacheInterval);
				header( "Pragma: cache"); // no-cache以外を設定

				header( 'ETag: "' . $etag . '"' );
				header( "Last-Modified: " . $sLastModified );
				
				// キャッシュを返す
				ADPageCache::outputCache( ADPageCache::createCacheIDFromRequest() );
			}
			
			return;
		}
	
		// OutputConfigの読み込み
		$siteConfig = new CMSSiteConfig();
		$outputConfig =& $siteConfig->getOutputConfig();	// インスタンスを変更する為、参照で取得する。
		
		// XSLTライブラリ番号の設定
		ADXslt::setXsltLibNo( $outputConfig->getXsltLibNo() );
		
		// XSLTによる文字エンコーディング処理の抑制の有無
		ADXslt::setDisableXsltEncoding( $outputConfig->getDisableXsltEncoding() );
		
		// 出力バッファを取得し、一旦出力をクリアする。
		$buffer = ob_get_contents();
		ob_end_clean();
		$pagedata = execute_cmsd_command( $buffer );
		
		// 出力文字コードの決定
		$outputEncoding = $outputConfig->getEncoding();	// execute_cmsd_commandメソッド内で、出力文字コードの設定がローカルに変更されている場合がある。
		
		// HTTPヘッダ出力用の文字列決定
		// 日本で良く使われるエンコーディングのみ対応
		$httpHeaderEncoding = ''; // 初期値はなし
		if ( strcasecmp( $outputEncoding, 'ISO-2022-JP' ) == 0 ) {
			$httpHeaderEncoding = 'ISO-2022-JP';
		}
		else if ( strcasecmp( $outputEncoding, 'Shift_JIS' ) == 0 ) {
			$httpHeaderEncoding = 'Shift_JIS';
		}
		else if ( strcasecmp( $outputEncoding, 'EUC-JP' ) == 0 ) {
			$httpHeaderEncoding = 'EUC-JP';
		}
		else if ( strcasecmp( $outputEncoding, 'UTF-8' ) == 0 ) {
			$httpHeaderEncoding = 'UTF-8';
		}
		
		// ヘッダにエンコーディングを出力
		// （直接 $outputEncodingを出力しない理由は、ヘッダへの攻撃可能性を極小化する為）
		if ( $httpHeaderEncoding != '' )
		{
			header( 'Content-Type: text/html; charset=' . $httpHeaderEncoding );
		}
		
		// METAタグ用出力文字コードの決定
		$metaEncoding = strtolower( $outputEncoding );
		if ( strcasecmp( $outputEncoding, 'shift_jis' ) == 0 or strcasecmp( $outputEncoding, 'sjis' ) == 0 )
		{
			$outputEncoding = 'SJIS-win'; // SJIS-win
			$metaEncoding = 'shift_jis';
		}
		
		// 文字コード変換
		if ( strcasecmp( $outputEncoding, gCMSDInternalEncoding() ) != 0 and strcasecmp( $outputEncoding, 'PASS' ) != 0 )	// EUC-JP or PASS の場合は文字コード変換を行わない。2015.08.28 UTF-8化対応で修正。
		{
			// metaタグ内のcharsetを置換する。
			if ( $outputConfig->getMetaTagReplacement() )
			{
				$pagedata = str_replace( 
					array( 	
						'charset=' . strtolower(gCMSDInternalEncoding()) . '"',
						'charset=' . strtoupper(gCMSDInternalEncoding()) . '"', 
						'charset="' . strtolower(gCMSDInternalEncoding()) . '"', 
						'charset="' . strtoupper(gCMSDInternalEncoding()) . '"' 
					), 
					array( 
						"charset={$metaEncoding}\"", 
						"charset={$metaEncoding}\"", 
						"charset=\"{$metaEncoding}\"", 
						"charset=\"{$metaEncoding}\"" 
					), 
					$pagedata 
				);
			}
		
			// 出力文字エンコードを変換する。
			$pagedata = mb_convert_encoding( $pagedata, $outputEncoding, gCMSDInternalEncodingPHP() ); // eucJP-win
		}
		
		// キャッシュが有効なら、キャッシュを生成する
		if ( ADPageCache::isEnabled() )
		{
			// キャッシュを生成
			ADPageCache::createCache( $cacheID, $pagedata );
			
			// キャッシュが存在していたら、そのキャッシュの日付をLast-Modifiedで返す。
			if ( ADPageCache::exists( $cacheID ) )
			{
				$timestamp = filemtime( ADPageCache::getCacheFilePath( $cacheID ) );
				$sLastModified = gmdate('D, d M Y H:i:s T', $timestamp);
				
				header( "Expires: " . gmdate( "D, d M Y H:i:s", time() + $cacheInterval ) . " GMT" );
				header( "Cache-Control: max-age=" . $cacheInterval);
				header( "Pragma: cache"); // no-cache以外を設定
				
				header('ETag: "' . $etag . '"' );
				header("Last-Modified: " . $sLastModified );
			}
		}
		
		// 出力
		print $pagedata;
	}
	
	function proc_cmsd_tag( $cmsdTag )
	{
		$result = '';
		
		// この処理の直前のタグ読み込み処理でcompilation errorがエラーバッファに溜まっており、ここで問題を起こす為、一旦エラー情報をクリアする。
		if ( function_exists( 'libxml_clear_errors' ) )
		{
			libxml_clear_errors();
		}
	
		//$old_error_reporting = error_reporting( E_ALL ^ ( E_NOTICE | E_WARNING ) );
		$errors = array();
		$xmlDocCommand = @domxml_open_mem('<?xml version="1.0"	encoding="UTF-8" ?><cmsdtag xmlns:cmsd="http://cms.al-design.jp/CMSDTAGS" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">' . mb_convert_encoding(	$cmsdTag, 'UTF-8', gCMSDInternalEncoding() ) . '</cmsdtag>', DOMXML_LOAD_PARSING, $errors );
		//error_reporting( $old_error_reporting );
		
		if( is_null( $xmlDocCommand ) or count( $errors ) > 0 ) {
			return "CMSD tag error. tag='" . htmlentities($cmsdTag) . "', errormessage = " . $errors[0]['errormessage'];
		}
		
		// Namespaceの為にダミーで付与したcmsdtag要素を排除する。
		$xmlDammyRoot = $xmlDocCommand->document_element();
		$xmlRoot = $xmlDammyRoot->first_child();
		
		//return "<font color=\"red\">TEST1:" . $xmlRoot->tagname() . "</font>";
		
		switch ( $xmlRoot->tagname() )
		{
			case 'entry' :
			case 'cmsd:entry' : // PHP5対応
				// エントリ名
				$name = $xmlRoot->get_attribute('name');				// エントリフォルダ名
				$eid = $xmlRoot->get_attribute('eid');					// エントリID
				$design = $xmlRoot->get_attribute('design');			// デザイン定義名
				$navigation = $xmlRoot->get_attribute('navigation');	// ナビゲーションのOn/Off
				$output = $xmlRoot->get_attribute('output');			// 出力形式 html/xml
				//$rows = $xmlRoot->get_attribute('rows');				// 1ページに出力する行数。
				$tagremoving = $xmlRoot->get_attribute('tagremoving');	// タグ除去
				//$cols = $xmlRoot->get_attribute('cols');				// テーブル形式での出力
				$dataname = $xmlRoot->get_attribute('dataname');		// 項目単位の出力指定
				$rows = $xmlRoot->get_attribute('rows');				// 1ページに出力する行数。2006.2.16 追加。
				$interlock = $xmlRoot->get_attribute('interlock');		// 一覧-詳細連動指定 'yes' | 'no'。2006.2.16 追加。
				
				// チェック：name未指定
				if ( strlen( $name ) <= 0 )
				{
					return "cmsd:entryタグにname属性が指定されていません";
				}
				
				// エントリフォルダ名、エントリIDを指定
				$v = new entry_view( $name, $eid );
				
				// タグ除去
				if ( $tagremoving == 'on' )
				{
					$v->tagRemovingOn();
				}
				
				//die( "interlock='{$interlock}', name='$name', dataname='$dataname'" );
				
				// 一覧-詳細連動指定。2006.2.16 追加。
				if ( $interlock == 'yes' )
				{
					$v->interlockOn();
				}
				
				// 1ページ最大出力件数
				if ( is_numeric( $rows ) )
				{
					$v->setPageMaxRows( $rows );
				}
								
				// ナビゲーションOn
				if ( $navigation == 'on' )
				{
					$v->navigationOn();
				}
				
				// グループキーを指定
				$xmlGroups = $xmlRoot->get_elements_by_tagname( 'group' );
				foreach( $xmlGroups as $xmlGroup )
				{
					$groupKey = $xmlGroup->get_attribute('key');
					$groupValue = $xmlGroup->get_attribute('value');
					$v->addGroupKey( $groupKey, $groupValue );
				}
				
				// パラメータキーを指定
				$xmlParams = $xmlRoot->get_elements_by_tagname( 'param' );
				foreach( $xmlParams as $xmlParam )
				{
					$paramKey = $xmlParam->get_attribute('key');
					$paramValue = $xmlParam->get_attribute('value');
					$v->addParamKey( $paramKey, $paramValue );
				}
				
				// インラインデザイン定義の指定
				// グループキーを指定
				$xmlDesigns = $xmlRoot->get_elements_by_tagname( 'design' );
				foreach( $xmlDesigns as $xmlDesign )
				{
					// design要素の子要素全てを文字列として取得し、inlineXsltにセット。
					$inlineXslt = '';
					foreach( $xmlDesign->child_nodes() as $xmlDesignChild )
					{
						$inlineXslt .= $xmlDocCommand->dump_node( $xmlDesignChild );
					}
					
					$v->setInlineXslt( $inlineXslt );
					
				}
				
				// 項目のみの出力か？
				if ( $dataname )
				{
					// 項目単位の出力を行う。
					return $v->getdata( $dataname );
				}
				else
				{
					// 通常のエントリ出力を行う。
					// xml形式？
					if ( $output == 'xml' )
					{
						// OutputConfigの読み込み
						$siteConfig = new CMSSiteConfig();
						$outputConfig =& $siteConfig->getOutputConfig();	// インスタンスを変更する為、参照で取得する。
						$outputConfig->setEncoding('pass');	// xml指定の場合、出力が既にUTF-8になっているので変換不要。
					
						// XML形式での出力
						return $v->getxml( $design );
					}
					else
					{
						return $v->gethtml( $design );
					}
				}
			
				break;
			
			case 'entrylist' :
			case 'cmsd:entrylist' : // PHP5対応

				// エントリ名
				$name = $xmlRoot->get_attribute('name');				// エントリフォルダ名
				$pageno = $xmlRoot->get_attribute('pageno');			// ページ番号
				$design = $xmlRoot->get_attribute('design');			// デザイン定義名
				//$navigation = $xmlRoot->get_attribute('navigation');	// ナビゲーションのOn/Off  ※エントリ一覧の場合、pageno='top'とすれば自動的にナビゲーションOffになるので不要。
				$output = $xmlRoot->get_attribute('output');			// 出力形式 html/xml
				$rows = $xmlRoot->get_attribute('rows');				// 1ページに出力する行数。
				$tagremoving = $xmlRoot->get_attribute('tagremoving');	// タグ除去
				$cols = $xmlRoot->get_attribute('cols');				// テーブル形式での出力
				$interlock = $xmlRoot->get_attribute('interlock');		// 一覧-詳細連動指定 'yes' | 'no'。2006.2.16 追加。
				
				// チェック：name未指定
				if ( strlen( $name ) <= 0 )
				{
					return "cmsd:entryタグにname属性が指定されていません";
				}
				
				// 初期化
				if ( empty( $rows ) )
				{
					$rows = 10;	// 規定値を設定
				}
				
				// エントリフォルダ名、ページ番号を指定
				$v = new entrylist_view( $name, $pageno );
				
				//$result .= 'entrylist tag <br />';
				
				// タグ除去
				if ( $tagremoving == 'on' )
				{
					$v->tagRemovingOn();
				}
				
				// 一覧-詳細連動指定。2006.2.16 追加。
				if ( $interlock == 'yes' )
				{
					$v->interlockOn();
				}
				
				// テーブル形式での出力
				if ( ! empty( $cols ) )
				{
					$v->setCols( $cols );
				}
				
				// グループキーを指定
				$xmlGroups = $xmlRoot->get_elements_by_tagname( 'group' );
				foreach( $xmlGroups as $xmlGroup )
				{
					$groupKey = $xmlGroup->get_attribute('key');
					$groupValue = $xmlGroup->get_attribute('value');
					/*
					if ( empty( $groupValue ) )
					{
						$groupValue = null;
					}
					*/
					$v->addGroupKey( $groupKey, $groupValue );
					//$result .= 'groupKey = ' . $groupKey . ', groupValue = ' . $groupValue . '<br />';
					
				}
				
				// パラメータキーを指定
				$xmlParams = $xmlRoot->get_elements_by_tagname( 'param' );
				foreach( $xmlParams as $xmlParam )
				{
					$paramKey = $xmlParam->get_attribute('key');
					$paramValue = $xmlParam->get_attribute('value');
					$v->addParamKey( $paramKey, $paramValue );
				}
				
				// ソートキーを指定
				$xmlSorts = $xmlRoot->get_elements_by_tagname( 'sort' );
				foreach( $xmlSorts as $index=>$xmlSort )
				{
					// ソートキーを指定
					$sortKey = $xmlSort->get_attribute('key');
					$sortOrder = $xmlSort->get_attribute('order');
					$sortKeyType = $xmlSort->get_attribute('type');
					$sortName = $xmlSort->get_attribute('name');
					
					if ( ! empty( $sortName ) )
					{
						// nameが指定されている場合は、URL指定なので他パラメータは無視する。
						$v->addSortKey( $sortName );
					}
					else
					{
						// nameが指定されていない場合は、各パラメータを指定する。第1パラメータは優先順位、第2パラメータは'key,order,type'の形式。
						$v->addSortKey( $index, $sortKey . ',' . $sortOrder . ',' . $sortKeyType );
					}
				}
				
				// インラインデザイン定義の指定
				// グループキーを指定
				$xmlDesigns = $xmlRoot->get_elements_by_tagname( 'design' );
				foreach( $xmlDesigns as $xmlDesign )
				{
					// design要素の子要素全てを文字列として取得し、inlineXsltにセット。
					$inlineXslt = '';
					foreach( $xmlDesign->child_nodes() as $xmlDesignChild )
					{
						$inlineXslt .= $xmlDocCommand->dump_node( $xmlDesignChild );
					}
					
					$v->setInlineXslt( $inlineXslt );
					
				}
				
				// OutputConfigの読み込み
				$siteConfig = new CMSSiteConfig();
				$outputConfig =& $siteConfig->getOutputConfig();	// インスタンスを変更する為、参照で取得する。
				
				// xml形式？
				if ( $output == 'xml' )
				{
					$outputConfig->setEncoding('pass');	// xml指定の場合、出力が既にUTF-8になっているので変換不要。
					$result .= $v->getxml( $design, $rows );
				}
				elseif ( $output == 'rss' )
				{
					$outputConfig->setEncoding('pass');	// rss指定の場合、出力はXSLT側で既にUTF-8になっているので変換不要。
					$result .= $v->getrss( $design );
				}
				else
				{
					$result .= $v->gethtml( $design, $rows );
				}
				
				break;
				
				
			case 'searchresult' :
			case 'cmsd:searchresult' : // PHP5対応
			
				// エントリ名
				$design = $xmlRoot->get_attribute('design');			// デザイン定義名
				$rows = $xmlRoot->get_attribute('rows');				// 1ページに出力する行数。2006.2.16 追加。
				
				// エントリフォルダ名、エントリIDを指定
				$v = new searchresult_view();
				
				// 対象エントリフォルダを指定
				$xmlTargets = $xmlRoot->get_elements_by_tagname( 'searchtarget' );
				
				if ( empty( $xmlTargets ) )
				{
					return "cmsd:searchresultタグ内にcmsd:searchtargetタグが1つもありません";
				}
				
				foreach( $xmlTargets as $xmlTarget )
				{
					$name = $xmlTarget->get_attribute('name');
					$title = $xmlTarget->get_attribute('title');
					$pageurl = $xmlTarget->get_attribute('pageurl');
					
					if ( strlen( $name ) <= 0 )
					{
						return "cmsd:searchtargetタグにname属性が指定されていません";
					}
					
					if ( strlen( $title ) <= 0 )
					{
						return "cmsd:searchtargetタグにtitle属性が指定されていません";
					}
					
					if ( strlen( $pageurl ) <= 0 )
					{
						return "cmsd:searchtargetタグにpageurl属性が指定されていません";
					}
					
					$v->addTarget( $name, $title, $pageurl );
				}
				
				// インラインデザイン定義の指定
				// グループキーを指定
				$xmlDesigns = $xmlRoot->get_elements_by_tagname( 'design' );
				foreach( $xmlDesigns as $xmlDesign )
				{
					// design要素の子要素全てを文字列として取得し、inlineXsltにセット。
					$inlineXslt = '';
					foreach( $xmlDesign->child_nodes() as $xmlDesignChild )
					{
						$inlineXslt .= $xmlDocCommand->dump_node( $xmlDesignChild );
					}
					
					$v->setInlineXslt( $inlineXslt );
					
				}
				
				return $v->output( $design, $rows );
			
				break;
				
			case 'xml' :
			case 'cmsd:xml' : // PHP5対応
			
				// 属性の読み込み
				$src = parseString( $xmlRoot->get_attribute('src'), null) ;			// XMLソースのパス
				$design = parseString( $xmlRoot->get_attribute('design'), null );	// デザイン定義名
				
				// チェック：src未指定
				if ( strlen( $src ) <= 0 )
				{
					return "cmsd:xmlタグにsrc属性が指定されていません";
				}
				
				// エントリフォルダ名、エントリIDを指定
				$v = new externalxml_view( $src );
				
				if ( is_null( $design ) )
				{
					$result = $v->output( null, true );
				}
				else
				{
					$result = $v->output( $design );
				}
			
				break;
				
			
			// ページローカルのエンコーディング設定
			case 'output' :
			case 'cmsd:output' : // PHP5対応
			
				$outputEncoding = parseString( $xmlRoot->get_attribute('encoding'), null) ;							// 出力文字エンコード指定（指定なしの場合null）
				$disableXsltEncoding = parseBoolean( $xmlRoot->get_attribute( 'disable-xslt-encoding' ), null );	// XSLTの文字エンコーディング無効化
				$metaTagReplacement = parseBoolean( $xmlRoot->get_attribute( 'meta-tag-replacement' ), null );		// メタタグのcharset置換
				$xsltLibNo = parseNumeric( $xmlRoot->get_attribute( 'xslt-libno' ), null );							// XSLT-Lib番号
				
				// XSLTライブラリ番号の設定
				if ( ! is_null( $xsltLibNo ) )
				{
					ADXslt::setXsltLibNo( $xsltLibNo );
				}
				
				// XSLTによる文字エンコーディング処理の抑制の有無
				if ( ! is_null( $disableXsltEncoding ) )
				{
					ADXslt::setDisableXsltEncoding( $disableXsltEncoding );
				}
				
				// OutputConfigの読み込み
				$siteConfig = new CMSSiteConfig();
				$outputConfig =& $siteConfig->getOutputConfig();	// インスタンスを変更する為、参照で取得する。
				
				// 出力文字コード指定
				if ( ! is_null( $outputEncoding ) )
				{
					$outputConfig->setEncoding( $outputEncoding );
				}
				
				// メタタグのcharset置換設定
				if ( ! is_null( $metaTagReplacement ) )
				{
					$outputConfig->setMetaTagReplacement( $metaTagReplacement );
				}
				
				break;
		
		}
		
		return $result;
		
		//return '<font color="green">' . $xmlRoot->tagname() . '</font>';
		
	
		//return '<font color="green">' . htmlentities( $cmsdTag ) . '</font>';
	}
	
	
	/* ---------------------------------------------------------------
	 * 
	 * $buffer : 出力バッファ内容
	 */	
	function execute_cmsd_command( $buffer )
	{
		$tagName = '';
		$output = '';
		$parsingText = '';
		$mode = 'text';
		
		// バッファを改行でsplitする。
		//$lineArr = explode( "\n", $buffer );
		//$lineIndex = 0;
		
		$parsingText = $buffer;
		
		while ( $parsingText != '' )
		{
		
			if ( $mode=='text' )
			{
				//print "mode = text<br />";
				preg_match( "/<cmsd:([a-z\-]+)/", $parsingText, $matches, PREG_OFFSET_CAPTURE );
				if ( $matches )	// マッチしたか？（マッチすると、$matches[0][0]にマッチ文字列全体が、$matches[0][1]にマッチ部分のオフセットが、$matches[1][0]には()部分の文字列、$matches[1][1]にはそのオフセットが入る。
				{
					// 見つかったら、タグの抽出を開始する。
					
					// タグ名を保持しておく。
					$tagName = $matches[1][0];
					
					$matchespos = mb_strpos( $parsingText, $matches[0][0] );	// substrを使うと、日本語を含む文字列をうまく処理できない事があるサーバの例が報告された為、mb系に置き換えた。matches[0][1]には文字列長ではなくバイト長が入る為、mb_strposを使って再検索する。
					//die( 'matchespos = ' . $matchespos . ', parse = ' . substr( $parsingText, $matchespos ) );
					
					$output .= mb_substr( $parsingText, 0, $matchespos ); //$matches[0][1] );  // pos以前の文字列。
					$parsingText = mb_substr( $parsingText, $matchespos ); //$matches[0][1] );	//pos以後の文字列。
					$mode = 'tag_starting';
					
					//print "tagName = $tagName<br />";
				}
				else
				{
					// 見つからなかったら、全て出力してしまう。
					$output .= $parsingText;
					$parsingText = '';
					
					//print "tag was not found.<br />";
				}
				
				
			}
			else if ( $mode=='tag_starting' )
			{
				//print '<span style="color:red">parsingText = "' . $parsingText . '"</span><br />';
				//print "mode = tag_starting<br />";
				preg_match( "/^(<cmsd:[^<]*\/>)/", $parsingText, $matches, PREG_OFFSET_CAPTURE );
				if ( $matches )
				{
					// 見つかったら、タグの抽出を終了し、タグを実行する。
					$cmsdTag = $matches[1][0];//substr( $parsingText, 0, $pos + 2 );  //pos+検出文字列 以前の文字列。
					
					// タグを実行。
					$output .= proc_cmsd_tag( $cmsdTag );
					
					// 残りをtextモードで再処理に回す。
					$parsingText = mb_substr( $parsingText, mb_strlen( $matches[1][0] ) ); // substr( $parsingText, $pos + 2 ); //pos+検出文字列 以後の文字列。
					$mode = 'text';
					
					//print "cmsdTag executed.<br />";
					//print " - " . htmlentities( $cmsdTag ) . "<br />";
				}
				else
				{
					preg_match( "/^(<cmsd:[^<]*>)/", $parsingText, $matches, PREG_OFFSET_CAPTURE );
					if ( $matches )
					{
						// 見つかったら、タグ開始済とし、タグの抽出を続行する。
						$mode = 'tag_started';
						//print "tagend.<br />";
					}
					else
					{
						// タグが閉じられておらず、エラーなので、エラーメッセージを出力して終了。
						$output .= "[ Error! '&lt;cmsd:$tagName&gt;' tag isn't ended. ]"; //$parsingTest;
						$parsingText = '';
						//print "error! tagend was not found.<br />";
					}
				}
			}
			else if ( $mode=='tag_started' )
			{
				//print "mode = tag_started<br />";
				$pos = mb_strpos( $parsingText, "</cmsd:$tagName>" );	//見つからなければstrposはfalseを返す。
				if ( $pos !== false )
				{
					// 見つかったら、タグの抽出を終了し、タグを実行する。
					$cmsdTag = mb_substr( $parsingText, 0, $pos + mb_strlen( "</cmsd:$tagName>" ) ); //pos+検出文字列 以前の文字列。
					
					// タグを実行。
					$output .= proc_cmsd_tag( $cmsdTag );
					
					// 残りをtextモードで再処理に回す。
					$parsingText = mb_substr( $parsingText, $pos + mb_strlen( "</cmsd:$tagName>" ) ); //pos+検出文字列 以後の文字列。
					$mode = 'text';
					
					//print "cmsdTag executed.<br />";
					//print " - " . htmlentities( $cmsdTag ) . "<br />";
				}
				else
				{
					// タグが閉じられておらず、エラーなので、エラーメッセージを出力して終了。
						$output .= "[ Error! '&lt;/cmsd:$tagName&gt;' closing-tag doesn't exist. ]"; //$parsingTest;
					$parsingText = '';

					//print "error! closetag was not found.<br />";
				}
				
			}
		
		}
		
		
		return $output;
  
	}



?>